#ifndef KBF_QUEUE_H
#define KBF_QUEUE_H

#include <optional>

#include <freertos/FreeRTOS.h>
#include <freertos/queue.h>

#include "kbf/exception.h"

namespace kbf {
    template <typename T>
    class Queue {
    public:
        explicit Queue(const unsigned int maxSize) : maxSize(maxSize) {
            static_assert(!std::is_base_of<std::string, T>::value, "std::string is not supported");
            if (!(handle = xQueueCreate(maxSize, sizeof(T)))) throw exception::OutOfMemoryError();
        }

        bool push(T value) {
            return xQueueSendToBack(handle, ((void *) &value), 0) == pdTRUE;
        }

        std::optional<T> pop() {
            T value;
            if (xQueueReceive(handle, &value, portMAX_DELAY) == pdTRUE) return value;
            else return std::nullopt;
        }

        std::optional<T> pop(int timeoutMs) {
            T value;
            if (xQueueReceive(handle, &value, timeoutMs / portTICK_PERIOD_MS) == pdTRUE) return value;
            else return std::nullopt;
        }

        int size() {
            return uxQueueMessagesWaiting(handle);
        }

        bool empty() {
            return uxQueueMessagesWaiting(handle) == 0;
        }

        void clear() {
            xQueueReset(handle);
        }

        const unsigned int maxSize;
    private:
        QueueHandle_t handle;
    };
}

#endif //KBF_QUEUE_H
